@@ -33,7 +33,7 @@ class _ClTransferData(CAMB_Structure):
                 ('delta_size', c_int * 3),
                 ('delta_p_l_k', POINTER(c_double)),
                 ('l_size', c_int),
-                ('l', POINTER(c_int))
+                ('L', POINTER(c_int))
                 ]
 
 
@@ -48,7 +48,8 @@ def save_cmb_power_array(filename, array, labels, lmin=0):
     lmax = array.shape[0] - 1
     ls = np.atleast_2d(np.arange(lmin, lmax + 1)).T
     ncol = array.shape[1]
-    if isinstance(labels, six.string_types): labels = labels.split()
+    if isinstance(labels, six.string_types):
+        labels = labels.split()
     np.savetxt(filename, np.hstack((ls, array[lmin:, :])), fmt=['%4u'] + ['%12.7e'] * ncol,
                header=' L ' + ' '.join(['{:13s}'.format(lab) for lab in labels]))
 
@@ -60,27 +61,30 @@ class MatterTransferData(object):
 
     To get an instance of this data, call :meth:`.results.CAMBdata.get_matter_transfer_data`.
 
-    For a description of the different Transfer_xxx outputs (and 21cm case) see :ref:`transfer-variables`.
+    For a description of the different Transfer_xxx outputs (and 21cm case) see :ref:`transfer-variables`; the
+    array is indexed by index+1 gven by:
+
+    - Transfer_kh = 1 (k/h)
+    - Transfer_cdm = 2 (cdm)
+    - Transfer_b = 3 (baryons)
+    - Transfer_g = 4 (photons)
+    - Transfer_r = 5 (massless neutrinos)
+    - Transfer_nu = 6 (massive neutrinos)
+    - Transfer_tot = 7 (total matter)
+    - Transfer_nonu = 8 (total matter excluding neutrinos)
+    - Transfer_tot_de = 9 (total including dark energy perturbations)
+    - Transfer_Weyl = 10 (Weyl potential)
+    - Transfer_Newt_vel_cdm = 11 (Newtonian CDM velocity)
+    - Transfer_Newt_vel_baryon = 12 (Newtonian baryon velocity)
+    - Transfer_vel_baryon_cdm = 13 (relative baryon-cdm velocity)
+
 
     :ivar nq:  number of q modes calculated
     :ivar q: array of q values calculated
     :ivar sigma_8: array of :math:`\sigma_8` values for each redshift
     :ivar sigma2_vdelta_8: array of v-delta8 correlation, so sigma2_vdelta_8/sigma_8 can define growth
-    :ivar transfer_data: numpy array T[entry, q_index, z_index] storing transfer functions for each redshift and q; entry+1 can be
-
-            - Transfer_kh = 1 (k/h)
-            - Transfer_cdm = 2 (cdm)
-            - Transfer_b = 3 (baryons)
-            - Transfer_g = 4 (photons)
-            - Transfer_r = 5 (massless neutrinos)
-            - Transfer_nu = 6 (massive neutrinos)
-            - Transfer_tot = 7 (total matter)
-            - Transfer_nonu = 8 (total matter excluding neutrinos)
-            - Transfer_tot_de = 9 (total including dark energy perturbations)
-            - Transfer_Weyl = 10 (Weyl potential)
-            - Transfer_Newt_vel_cdm = 11 (Newtonian CDM velocity)
-            - Transfer_Newt_vel_baryon = 12 (Newtonian baryon velocity)
-            - Transfer_vel_baryon_cdm = 13 (relative baryon-cdm velocity)
+    :ivar transfer_data: numpy array T[entry, q_index, z_index] storing transfer functions for each redshift and q;
+                         entry+1 can be one of the Transfer_xxx variables above.
     """
 
     def transfer_z(self, name, z_index=0):
@@ -92,7 +96,7 @@ class MatterTransferData(object):
         :return: array of transfer function values for each calculated k
         """
 
-        if not name in model.transfer_names:
+        if name not in model.transfer_names:
             raise CAMBError('Unknown name %s; must be one of %s' % (name, model.transfer_names))
         return self.transfer_data[model.transfer_names.index(name), :, z_index]
 
@@ -104,8 +108,8 @@ class ClTransferData(object):
 
     :ivar NumSources:  number of sources calculated (size of p index)
     :ivar q: array of q values calculated (=k in flat universe)
-    :ivar l: int array of :math:`\ell` values calculated
-    :ivar delta_p_l_k: transfer functions, indexed by source, l, q
+    :ivar L: int array of :math:`\ell` values calculated
+    :ivar delta_p_l_k: transfer functions, indexed by source, L, q
     """
 
     def get_transfer(self, source=0):
@@ -114,7 +118,7 @@ class ClTransferData(object):
         and :math:`q` (:math:`= k` in a flat universe).
 
         :param source: index of source: e.g. 0 for temperature, 1 for E polarization, 2 for lensing potential
-        :return: array of computed l, array of computed q, transfer functions T(l,q)
+        :return: array of computed L, array of computed q, transfer functions T(L,q)
         """
 
         return self.l, self.q, self.delta_p_l_k[source, :, :]
@@ -161,11 +165,11 @@ class CAMBdata(F2003Class):
                 ("Ksign", c_double, "Ksign = 1,0 or -1"),
                 ("tau0", c_double, "conformal time today"),
                 ("chi0", c_double, "comoving angular diameter distance of big bang; rofChi(tau0/curvature_radius)"),
-                ("scale", c_double, "relative to flat. e.g. for scaling l sampling"),
+                ("scale", c_double, "relative to flat. e.g. for scaling L sampling"),
                 ("akthom", c_double, "sigma_T * (number density of protons now)"),
                 ("fHe", c_double, "n_He_tot / n_H_tot"),
                 ("Nnow", c_double, "number density today"),
-                ("z_eq", c_double, "matter-radiation equality refshift assuming all neutrinos relativistic"),
+                ("z_eq", c_double, "matter-radiation equality redshift assuming all neutrinos relativistic"),
                 ("grhormass", c_double * model.max_nu),
                 ("nu_masses", c_double * model.max_nu),
                 ("num_transfer_redshifts", c_int,
@@ -195,7 +199,7 @@ class CAMBdata(F2003Class):
                  ('DarkEnergyStressEnergy', [numpy_1d, numpy_1d, numpy_1d, int_arg]),
                  ('get_lmax_lensed', [], c_int),
                  ('get_zstar', [d_arg], c_double),
-                 ('SetParams', [POINTER(CAMBparams), int_arg, int_arg, int_arg])
+                 ('SetParams', [POINTER(CAMBparams), int_arg, int_arg, int_arg, int_arg])
                  ]
 
     def __init__(self):
@@ -228,8 +232,8 @@ class CAMBdata(F2003Class):
         """
         n = len(self.Params.z_outputs)
         if not n:
-            raise CAMBError(
-                'Set z_outputs with required redshifts (and then calculate transfers/results) before calling get_background_outputs')
+            raise CAMBError('Set z_outputs with required redshifts (and then calculate transfers/results)'
+                            ' before calling get_background_outputs')
         outputs = np.empty((n, 4))
         CAMB_GetBackgroundOutputs(byref(self), outputs, byref(c_int(n)))
         return outputs
@@ -254,15 +258,16 @@ class CAMBdata(F2003Class):
         if not params.ombh2:
             raise CAMBValueError('Parameter values not set')
 
-    def calc_background_no_thermo(self, params):
+    def calc_background_no_thermo(self, params, do_reion=False):
         """
-        Calculate the background evolution without calculating thermal history.
+        Calculate the background evolution without calculating thermal or ionization history.
         e.g. call this if you want to just use :meth:`angular_diameter_distance` and similar background functions
 
         :param params:  :class:`~.model.CAMBparams` instance to use
+        :param do_reion: whether to initialize the reionization model
         """
         self._check_params(params)
-        self.f_SetParams(byref(params), None, None, None)
+        self.f_SetParams(byref(params), None, byref(c_int(1 if do_reion else 0)), None, byref(c_int(1)))
         config.check_global_error('calc_background_no_thermo')
 
     def calc_background(self, params):
@@ -278,19 +283,22 @@ class CAMBdata(F2003Class):
 
     def calc_transfers(self, params, only_transfers=True):
         """
-        Calculate the transfer functions (for CMB and matter power, as determined by params.WantCls, params.WantTransfer)
+        Calculate the transfer functions (for CMB and matter power, as determined
+        by params.WantCls, params.WantTransfer).
 
         :param params: :class:`~.model.CAMBparams` instance with parameters to use
         :param only_transfers: only calculate transfer functions, no power spectra
         :return: non-zero if error, zero if OK
         """
         self._check_params(params)
-        if not only_transfers: self._check_powers(params)
+        if not only_transfers:
+            self._check_powers(params)
         if CAMBdata_gettransfers(byref(self), byref(params), byref(c_int(1 if only_transfers else 0))):
             config.check_global_error('calc_transfer')
 
     def _check_powers(self, params=None):
-        if params is None: params = self.Params
+        if params is None:
+            params = self.Params
         if params.InitPower.has_tensors() and not params.WantTensors:
             raise CAMBError('r>0 but params.WantTensors = F')
         if params.WantScalars and params.WantCls and params.DoLensing and params.scalar_power(0.05) > 2e-8:
@@ -312,12 +320,12 @@ class CAMBdata(F2003Class):
 
     def power_spectra_from_transfer(self, initial_power_params, silent=False):
         """
-        Assuming :meth:`calc_transfers` or :meth:`calc_power_spectra` have already been used, re-calculate the power spectra
-        using a new set of initial power spectrum parameters with otherwise the same cosmology.
+        Assuming :meth:`calc_transfers` or :meth:`calc_power_spectra` have already been used, re-calculate the
+        power spectra using a new set of initial power spectrum parameters with otherwise the same cosmology.
         This is typically much faster that re-calculating everything, as the transfer functions can be re-used.
-        NOTE: if non-linear lensing is on, the transfer functions have the non-linear correction included when they are calculated, so
-        using this function with a different initial power spectrum will not give quite the same results as doing a
-        full recalculation.
+        NOTE: if non-linear lensing is on, the transfer functions have the non-linear correction included when
+        they are calculated, so using this function with a different initial power spectrum will not give quite the
+        same results as doing a full recalculation.
 
         :param initial_power_params: :class:`.initialpower.InitialPowerLaw` or :class:`.initialpower.SplinedInitialPower`
                instance with new primordial power spectrum parameters
@@ -377,11 +385,13 @@ class CAMBdata(F2003Class):
 
     def save_cmb_power_spectra(self, filename, lmax=None, CMB_unit='muK'):
         r"""
-        Save CMB power to a plain text file. Output is lensed total :math:`\ell(\ell+1)C_\ell/2\pi` then lensing potential and cross: L TT EE BB TE PP PT PE.
+        Save CMB power to a plain text file. Output is lensed total :math:`\ell(\ell+1)C_\ell/2\pi` then
+        lensing potential and cross: L TT EE BB TE PP PT PE.
 
         :param filename: filename to save
         :param lmax: lmax to save
-        :param CMB_unit: scale results from dimensionless. Use 'muK' for :math:`\mu K^2` units for CMB :math:`C_\ell` and :math:`\mu K` units for lensing cross.
+        :param CMB_unit: scale results from dimensionless. Use 'muK' for :math:`\mu K^2` units for CMB :math:`C_\ell`
+        and :math:`\mu K` units for lensing cross.
         """
         lmax = self._lmax_setting(lmax)
         cmb = self.get_total_cls(lmax, CMB_unit=CMB_unit)
@@ -389,18 +399,21 @@ class CAMBdata(F2003Class):
         save_cmb_power_array(filename, np.hstack((cmb, lens)), 'TT EE BB TE PP PT PE')
 
     def get_cmb_power_spectra(self, params=None, lmax=None,
-                              spectra=['total', 'unlensed_scalar', 'unlensed_total', 'lensed_scalar', 'tensor',
-                                       'lens_potential'], CMB_unit=None, raw_cl=False):
+                              spectra=('total', 'unlensed_scalar', 'unlensed_total', 'lensed_scalar', 'tensor',
+                                       'lens_potential'), CMB_unit=None, raw_cl=False):
         r"""
-        Get CMB power spectra, as requested by the 'spectra' argument. All power spectra are :math:`\ell(\ell+1)C_\ell/2\pi` self-owned
-        numpy arrays (0..lmax, 0..3), where 0..3 index are TT, EE, BB, TE, unless raw_cl is True in which case return just :math:`C_\ell`.
+        Get CMB power spectra, as requested by the 'spectra' argument. All power spectra are
+         :math:`\ell(\ell+1)C_\ell/2\pi` self-owned numpy arrays (0..lmax, 0..3), where 0..3 index
+         are TT, EE, BB, TE, unless raw_cl is True in which case return just :math:`C_\ell`.
         For the lens_potential the power spectrum returned is that of the deflection.
 
         :param params: optional :class:`~.model.CAMBparams` instance with parameters to use. If None, must have
-          previously set parameters and called `calc_power_spectra` (e.g. if you got this instance using :func:`.camb.get_results`),
-        :param lmax: maximum l
+          previously set parameters and called `calc_power_spectra` (e.g. if you got this instance
+          using :func:`.camb.get_results`),
+        :param lmax: maximum L
         :param spectra: list of names of spectra to get
-        :param CMB_unit: scale results from dimensionless. Use 'muK' for :math:`\mu K^2` units for CMB :math:`C_\ell` and :math:`\mu K` units for lensing cross.
+        :param CMB_unit: scale results from dimensionless. Use 'muK' for :math:`\mu K^2` units for CMB :math:`C_\ell`
+          and :math:`\mu K` units for lensing cross.
         :param raw_cl: return :math:`C_\ell` rather than :math:`\ell(\ell+1)C_\ell/2\pi`
         :return: dictionary of power spectrum arrays, indexed by names of requested spectra
         """
@@ -422,7 +435,8 @@ class CAMBdata(F2003Class):
         If xvals is explicitly given, instead calculates correlations at provided :math:`\cos(\theta)` values.
 
         :param params: optional :class:`~.model.CAMBparams` instance with parameters to use. If None, must have
-          previously set parameters and called :meth:`calc_power_spectra` (e.g. if you got this instance using :func:`.camb.get_results`),
+          previously set parameters and called :meth:`calc_power_spectra` (e.g. if you got this instance
+          using :func:`.camb.get_results`),
         :param lmax: optional maximum L to use from the cls arrays
         :param spectrum: type of CMB power spectrum to get; default 'lensed_scalar', one of
           ['total', 'unlensed_scalar', 'unlensed_total', 'lensed_scalar', 'tensor']
@@ -475,7 +489,8 @@ class CAMBdata(F2003Class):
 
         old_boost = self.Params.Accuracy.lAccuracyBoost
         try:
-            if lAccuracyBoost: self.Params.Accuracy.lAccuracyBoost = lAccuracyBoost
+            if lAccuracyBoost:
+                self.Params.Accuracy.lAccuracyBoost = lAccuracyBoost
             if not isinstance(vars, (tuple, list)):
                 vars = [vars]
             import sympy
@@ -551,7 +566,8 @@ class CAMBdata(F2003Class):
         :return: n_eta x len(vars) 2D numpy array of outputs or dict of 1D arrays
         """
 
-        if isinstance(vars, six.string_types): vars = [vars]
+        if isinstance(vars, six.string_types):
+            vars = [vars]
         unknown = set(vars) - set(model.background_names)
         if unknown:
             raise CAMBError('Unknown names %s; valid names are %s' % (unknown, model.background_names))
@@ -590,7 +606,8 @@ class CAMBdata(F2003Class):
         :param format: 'dict' or 'array', for either dict of 1D arrays indexed by name, or 2D array
         :return: n_a x len(vars) 2D numpy array or dict of 1D arrays of :math:`8\pi G a^4 \rho_i` in Mpc units.
         """
-        if isinstance(vars, six.string_types): vars = [vars]
+        if isinstance(vars, six.string_types):
+            vars = [vars]
         unknown = set(vars) - set(model.density_names)
         if unknown:
             raise CAMBError('Unknown names %s; valid names are %s' % (unknown, model.density_names))
@@ -663,27 +680,35 @@ class CAMBdata(F2003Class):
 
     @staticmethod
     def _transfer_var(var1, var2):
-        if var1 is None: var1 = config.transfer_power_var
-        if var2 is None: var2 = config.transfer_power_var
-        if isinstance(var1, six.string_types): var1 = model.transfer_names.index(var1) + 1
-        if isinstance(var2, six.string_types): var2 = model.transfer_names.index(var2) + 1
+        if var1 is None:
+            var1 = config.transfer_power_var
+        if var2 is None:
+            var2 = config.transfer_power_var
+        if isinstance(var1, six.string_types):
+            var1 = model.transfer_names.index(var1) + 1
+        if isinstance(var2, six.string_types):
+            var2 = model.transfer_names.index(var2) + 1
         return c_int(var1), c_int(var2)
 
-    def get_linear_matter_power_spectrum(self, var1=None, var2=None,
-                                         hubble_units=True, have_power_spectra=True, params=None, nonlinear=False):
+    def get_linear_matter_power_spectrum(self, var1=None, var2=None, hubble_units=True, k_hunit=True,
+                                         have_power_spectra=True, params=None, nonlinear=False):
         r"""
-        Calculates :math:`P_{xy}(k/h)`, where x, y are one of model.Transfer_cdm, model.Transfer_xx etc.
-        The output k values are not regularly spaced, and not interpolated.
+        Calculates :math:`P_{xy}(k)`, where x, y are one of model.Transfer_cdm, model.Transfer_xx etc.
+        The output k values are not regularly spaced, and not interpolated. They are either k or k/h depending on the
+        value of k_hunit (default True gives k/h).
 
         For a description of outputs for different var1, var2 see :ref:`transfer-variables`.
 
         :param var1: variable i (index, or name of variable; default delta_tot)
         :param var2: variable j (index, or name of variable; default delta_tot)
         :param hubble_units: if true, output power spectrum in (Mpc/h) units, otherwise Mpc
+        :param k_hunit: if true, matter power is a function of k/h, if false, just k (both :math:`{\rm Mpc}^{-1}` units)
         :param have_power_spectra: set to False if not already computed power spectra
-        :param params: if have_power_spectra=False, optional :class:`~.model.CAMBparams` instance to specify new parameters
+        :param params: if have_power_spectra=False, optional :class:`~.model.CAMBparams` instance
+                       to specify new parameters
         :param nonlinear: include non-linear correction from halo model
-        :return: kh, z, PK, where kz an z are arrays of k/h and z respectively, and PK[i,j] is value at z[i], k/h[j]
+        :return: k/h or k, z, PK, where kz an z are arrays of k/h or k and z respectively,
+                 and PK[i,j] is the value at z[i], k[j]/h or k[j]
         """
         if self.OnlyTransfers or params is not None or not have_power_spectra:
             self.calc_power_spectra(params)
@@ -692,6 +717,8 @@ class CAMBdata(F2003Class):
         nk = data.nq
         nz = self.Params.Transfer.PK_num_redshifts
         kh = data.transfer_data[model.Transfer_kh - 1, :, 0]
+        if not k_hunit:
+            kh *= self.Params.H0 / 100
 
         var1, var2 = self._transfer_var(var1, var2)
 
@@ -707,7 +734,8 @@ class CAMBdata(F2003Class):
         z.reverse()
         return np.array(kh), np.array(z), PK
 
-    def get_nonlinear_matter_power_spectrum(self, **kwargs):
+    def get_nonlinear_matter_power_spectrum(self, var1=None, var2=None, hubble_units=True, k_hunit=True,
+                                            have_power_spectra=True, params=None):
         r"""
         Calculates :math:`P_{xy}(k/h)`, where x, y are one of model.Transfer_cdm, model.Transfer_xx etc.
         The output k values are not regularly spaced, and not interpolated.
@@ -716,13 +744,18 @@ class CAMBdata(F2003Class):
 
         :param var1: variable i (index, or name of variable; default delta_tot)
         :param var2: variable j (index, or name of variable; default delta_tot)
-        :param hubble_units: if true, output power spectrum in :Math:`({\rm Mpc}/h)^{3}` units, otherwise :math:`{\rm Mpc}^{3}`
+        :param hubble_units: if true, output power spectrum in :Math:`({\rm Mpc}/h)^{3}` units,
+                             otherwise :math:`{\rm Mpc}^{3}`
+        :param k_hunit: if true, matter power is a function of k/h, if false, just k (both :math:`{\rm Mpc}^{-1}` units)
         :param have_power_spectra: set to False if not already computed power spectra
-        :param params: if have_power_spectra=False, optional :class:`~.model.CAMBparams` instance to specify new parameters
-        :return: kh, z, PK, where kz an z are arrays of k/h and z respectively, and PK[i,j] is value at z[i], k/h[j]
+        :param params: if have_power_spectra=False, optional :class:`~.model.CAMBparams` instance
+                       to specify new parameters
+        :return: k/h or k, z, PK, where kz an z are arrays of k/h or k and z respectively,
+                 and PK[i,j] is the value at z[i], k[j]/h or k[j]
         """
-        kwargs['nonlinear'] = True
-        return self.get_linear_matter_power_spectrum(**kwargs)
+        return self.get_linear_matter_power_spectrum(var1=var1, var2=var2, hubble_units=hubble_units, k_hunit=k_hunit,
+                                                     have_power_spectra=have_power_spectra, params=params,
+                                                     nonlinear=True)
 
     def get_sigma8(self):
         r"""
@@ -759,7 +792,8 @@ class CAMBdata(F2003Class):
         :param var1: variable i (index, or name of variable; default delta_tot)
         :param var2: variable j (index, or name of variable; default delta_tot)
         :param have_power_spectra: set to True if already computed power spectra
-        :param params: if have_power_spectra=False and want to specify new parameters, a :class:`~.model.CAMBparams` instance
+        :param params: if have_power_spectra=False and want to specify new parameters,
+                       a :class:`~.model.CAMBparams` instance
         :return: kh, z, PK, where kz an z are arrays of k/h and z respectively, and PK[i,j] is value at z[i], k/h[j]
         """
 
@@ -805,15 +839,18 @@ class CAMBdata(F2003Class):
         :param nonlinear: include non-linear correction from halo model
         :param var1: variable i (index, or name of variable; default delta_tot)
         :param var2: variable j (index, or name of variable; default delta_tot)
-        :param hubble_units: if true, output power spectrum in :math:`({\rm Mpc}/h)^{3}` units, otherwise :math:`{\rm Mpc}^{3}`
+        :param hubble_units: if true, output power spectrum in :math:`({\rm Mpc}/h)^{3}` units,
+                             otherwise :math:`{\rm Mpc}^{3}`
         :param k_hunit: if true, matter power is a function of k/h, if false, just k (both :math:`{\rm Mpc}^{-1}` units)
         :param return_z_k: if true, return interpolator, z, k where z, k are the grid used
-        :param log_interp: if true, interpolate log of power spectrum (unless any values are negative in which case ignored)
-        :param extrap_kmax: if set, use power law extrapolation beyond kmax to extrap_kmax (useful for tails of integrals)
+        :param log_interp: if true, interpolate log of power spectrum
+                           (unless any values cross zero in which case ignored)
+        :param extrap_kmax: if set, use power law extrapolation beyond kmax to extrap_kmax
+                            (useful for tails of integrals)
         :param silent: Set True to silence warnings
-        :return: An object PK based on :class:`~scipy:scipy.interpolate.RectBivariateSpline`, that can be called with PK.P(z,kh)
-           or PK(z,log(kh)) to get log matter power values.
-           If return_z_k=True, instead return interpolator, z, k where z, k are the grid used.
+        :return: An object PK based on :class:`~scipy:scipy.interpolate.RectBivariateSpline`,
+                 that can be called with PK.P(z,kh) or PK(z,log(kh)) to get log matter power values.
+                 If return_z_k=True, instead return interpolator, z, k where z, k are the grid used.
         """
 
         class PKInterpolator(RectBivariateSpline):
@@ -822,7 +859,7 @@ class CAMBdata(F2003Class):
                 if grid is None:
                     grid = not np.isscalar(z) and not np.isscalar(kh)
                 if self.islog:
-                    return np.exp(self(z, np.log(kh), grid=grid))
+                    return self.logsign * np.exp(self(z, np.log(kh), grid=grid))
                 else:
                     return self(z, np.log(kh), grid=grid)
 
@@ -835,7 +872,7 @@ class CAMBdata(F2003Class):
             def check_z(self, z):
                 if not np.allclose(z, self._single_z):
                     raise CAMBError(
-                        "P(z,k) requested at z=%r, but only computed for z=%g. "
+                        "P(z,k) requested at z=%g, but only computed for z=%s. "
                         "Cannot extrapolate!" % (z, self._single_z))
 
             def __call__(self, *args):
@@ -847,7 +884,7 @@ class CAMBdata(F2003Class):
             def P(self, z, kh, grid=None):
                 # grid kwarg is ignored
                 if self.islog:
-                    return np.exp(self(z, np.log(kh)))
+                    return self.logsign * np.exp(self(z, np.log(kh)))
                 else:
                     return self(z, np.log(kh))
 
@@ -856,42 +893,44 @@ class CAMBdata(F2003Class):
         kh_max = kh[-1]
         if not k_hunit:
             kh *= self.Params.H0 / 100
+        sign = 1
         if log_interp and np.any(pk <= 0):
-            log_interp = False
+            if np.all(pk < 0):
+                sign = -1
+            else:
+                log_interp = False
+        p_or_log_p = np.log(sign * pk) if log_interp else pk
         logkh = np.log(kh)
         deg_z = min(len(z) - 1, 3)
+        kmax = kh[-1]
         PKInterpolator = PKInterpolator if deg_z else PKInterpolatorSingleZ
-        if extrap_kmax and extrap_kmax > kh[-1]:
+        if extrap_kmax and extrap_kmax > kmax:
             # extrapolate to ultimate power law
             # TODO: use more physical extrapolation function for linear case
             if not silent and (kh_max < 3 and extrap_kmax > 2 and nonlinear or kh_max < 0.4):
                 logging.warning("Extrapolating to higher k with matter transfer functions "
                                 "only to k=%.3g Mpc^{-1} may be inaccurate.\n " % (kh_max * self.Params.H0 / 100))
+            if not log_interp:
+                raise CAMBValueError(
+                    "Cannot use extrap_kmax with log_inter=False (e.g. PK crossing zero for %s, %s.)" % (var1, var2))
+
             logextrap = np.log(extrap_kmax)
-            logpknew = np.empty((pk.shape[0], pk.shape[1] + 2))
-            logpknew[:, :-2] = np.log(pk)
+            log_p_new = np.empty((pk.shape[0], pk.shape[1] + 2))
+            log_p_new[:, :-2] = p_or_log_p
             delta = logextrap - logkh[-1]
-            dlog = (logpknew[:, -3] - logpknew[:, -4]) / (logkh[-1] - logkh[-2])
-            logpknew[:, -1] = logpknew[:, -3] + dlog * delta
-            logpknew[:, -2] = logpknew[:, -3] + dlog * delta * 0.9
-            logkhnew = np.hstack((logkh, logextrap - delta * 0.1, logextrap))
 
-            deg_k = min(len(logkhnew) - 1, 3)
-            if log_interp:
-                res = PKInterpolator(z, logkhnew, logpknew, kx=deg_z, ky=deg_k)
-            else:
-                res = PKInterpolator(z, logkhnew, np.exp(logpknew), kx=deg_z, ky=deg_k)
-            res.kmax = extrap_kmax
-        else:
-            deg_k = min(len(logkh) - 1, 3)
-            if log_interp:
-                res = PKInterpolator(z, logkh, np.log(pk), kx=deg_z, ky=deg_k)
-            else:
-                res = PKInterpolator(z, logkh, pk, kx=deg_z, ky=deg_k)
-            res.kmax = np.max(kh)
+            dlog = (log_p_new[:, -3] - log_p_new[:, -4]) / (logkh[-1] - logkh[-2])
+            log_p_new[:, -1] = log_p_new[:, -3] + dlog * delta
+            log_p_new[:, -2] = log_p_new[:, -3] + dlog * delta * 0.9
+            logkh = np.hstack((logkh, logextrap - delta * 0.1, logextrap))
+            p_or_log_p = log_p_new
 
+        deg_k = min(len(logkh) - 1, 3)
+        res = PKInterpolator(z, logkh, p_or_log_p, kx=deg_z, ky=deg_k)
         res.kmin = np.min(kh)
+        res.kmax = kmax
         res.islog = log_interp
+        res.logsign = sign
         res.zmin = np.min(z)
         res.zmax = np.max(z)
         if return_z_k:
@@ -960,8 +999,7 @@ class CAMBdata(F2003Class):
         :return: numpy array CL[0:lmax+1,0:4], where 0..3 indexes TT, EE, BB, TE.
         """
         lmax = self._lmax_setting(lmax, unlensed=True)
-        return self.get_unlensed_scalar_cls(lmax, CMB_unit, raw_cl) + \
-               self.get_tensor_cls(lmax, CMB_unit, raw_cl)
+        return self.get_unlensed_scalar_cls(lmax, CMB_unit, raw_cl) + self.get_tensor_cls(lmax, CMB_unit, raw_cl)
 
     def get_lensed_scalar_cls(self, lmax=None, CMB_unit=None, raw_cl=False):
         r"""
@@ -982,7 +1020,8 @@ class CAMBdata(F2003Class):
 
     def get_lens_potential_cls(self, lmax=None, CMB_unit=None, raw_cl=False):
         r"""
-        Get lensing deflection angle potential power spectrum, and cross-correlation with T and E. Must have already calculated power spectra.
+        Get lensing deflection angle potential power spectrum, and cross-correlation with T and E. Must have already
+        calculated power spectra.
         Power spectra are :math:`[L(L+1)]^2C_L^{\phi\phi}/2\pi` and corresponding deflection cross-correlations.
 
         :param lmax: lmax to output to
@@ -1000,7 +1039,7 @@ class CAMBdata(F2003Class):
         return res
 
     def get_unlensed_scalar_array_cls(self, lmax=None):
-        """
+        r"""
         Get array of all cross power spectra. Must have already calculated power spectra.
         Results are dimensionless, and not scaled by custom_scaled_ell_fac.
 
@@ -1019,12 +1058,15 @@ class CAMBdata(F2003Class):
 
     def get_cmb_unlensed_scalar_array_dict(self, params=None, lmax=None, CMB_unit=None, raw_cl=False):
         r"""
-        Get all unlensed auto and cross power spectra, including any custom source functions set using :meth:`.model.CAMBparams.set_custom_scalar_sources`.
+        Get all unlensed auto and cross power spectra, including any custom source functions set
+        using :meth:`.model.CAMBparams.set_custom_scalar_sources`.
 
         :param params: optional :class:`~.model.CAMBparams` instance with parameters to use. If None, must have
-          previously set parameters and called :meth:`calc_power_spectra` (e.g. if you got this instance using :func:`.camb.get_results`),
+          previously set parameters and called :meth:`calc_power_spectra` (e.g. if you got this instance
+          using :func:`.camb.get_results`),
         :param lmax: maximum :math:`\ell`
-        :param CMB_unit: scale results from dimensionless. Use 'muK' for :math:`\mu K^2` units for CMB :math:`C_\ell` and :math:`\mu K` units for lensing cross.
+        :param CMB_unit: scale results from dimensionless. Use 'muK' for :math:`\mu K^2` units for
+          CMB :math:`C_\ell` and :math:`\mu K` units for lensing cross.
         :param raw_cl: return :math:`C_\ell` rather than :math:`\ell(\ell+1)C_\ell/2\pi`
         :return: dictionary of power spectrum arrays, index as TxT, TxE, PxW1, W1xW2, custom_name_1xT... etc.
                  Note that P is the lensing deflection, lensing windows Wx give convergence.
@@ -1072,12 +1114,13 @@ class CAMBdata(F2003Class):
         return result
 
     def get_source_cls_dict(self, params=None, lmax=None, raw_cl=False):
-        """
+        r"""
         Get all source window function and CMB lensing and cross power spectra. Does not include CMB spectra.
         Note that P is the deflection angle, but lensing windows return the kappa power.
 
         :param params: optional :class:`~.model.CAMBparams` instance with parameters to use. If None, must have
-          previously set parameters and called :meth:`calc_power_spectra` (e.g. if you got this instance using :func:`.camb.get_results`),
+          previously set parameters and called :meth:`calc_power_spectra`
+          (e.g. if you got this instance using :func:`.camb.get_results`),
         :param lmax: maximum :math:`\ell`
         :param raw_cl: return :math:`C_\ell` rather than :math:`\ell(\ell+1)C_\ell/2\pi`
         :return: dictionary of power spectrum arrays, index as PXP, PxW1, W1xW2, ... etc.
@@ -1119,16 +1162,19 @@ class CAMBdata(F2003Class):
 
     def get_lensed_gradient_cls(self, lmax=None, CMB_unit=None, raw_cl=False):
         r"""
-        Get lensed gradient scalar CMB power spectra in flat sky approximation (`arXiv:1101.2234 <https://arxiv.org/abs/1101.2234>`_).
-        Note that lmax used to calculate results may need to be substantially larger than the lmax output from this function
-        (there is no extrapolation as in the main lensing routines). Lensed power spectra must be already calculated.
+        Get lensed gradient scalar CMB power spectra in flat sky approximation
+        (`arXiv:1101.2234 <https://arxiv.org/abs/1101.2234>`_).
+        Note that lmax used to calculate results may need to be substantially larger than the lmax
+        output from this function (there is no extrapolation as in the main lensing routines).
+        Lensed power spectra must be already calculated.
 
         :param lmax: lmax to output to
         :param CMB_unit: scale results from dimensionless. Use 'muK' for :math:`\mu K^2` units for CMB :math:`C_\ell`
         :param raw_cl: return :math:`C_\ell` rather than :math:`\ell(\ell+1)C_\ell/2\pi`
-        :return: numpy array CL[0:lmax+1,0:8], where CL[:,i] are :math:`T\nabla T`, :math:`E\nabla E`, :math:`B\nabla B`, :math:`PP_\perp`,
-                 :math:`T\nabla E`, :math:`TP_\perp`, :math:`(\nabla T)^2`, :math:`\nabla T\nabla T` where the first six are as defined in
-                 appendix C of `1101.2234 <https://arxiv.org/abs/1101.2234>`_.
+        :return: numpy array CL[0:lmax+1,0:8], where CL[:,i] are :math:`T\nabla T`, :math:`E\nabla E`,
+                 :math:`B\nabla B`, :math:`PP_\perp`, :math:`T\nabla E`, :math:`TP_\perp`, :math:`(\nabla T)^2`,
+                 :math:`\nabla T\nabla T` where the first six are as defined in appendix C
+                 of `1101.2234 <https://arxiv.org/abs/1101.2234>`_.
         """
         assert self.Params.DoLensing
         lmax = self._lmax_setting(lmax)
@@ -1144,7 +1190,8 @@ class CAMBdata(F2003Class):
         """
         Get (non-comoving) angular diameter distance to redshift z.
 
-        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions or power spectra.
+        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer
+        functions or power spectra.
 
         :param z: redshift or array of redshifts
         :return: angular diameter distances, matching rank of z
@@ -1166,7 +1213,8 @@ class CAMBdata(F2003Class):
         :math:`\frac{r}{1+z_2}\text{sin}_K\left(\frac{\chi(z_2) - \chi(z_1)}{r}\right)`
         where :math:`r` is curvature radius and :math:`\chi` is the comoving radial distance.
 
-        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions or power spectra.
+        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer
+        functions or power spectra.
 
         :param z1: redshift 1
         :param z2: redshift 2
@@ -1180,7 +1228,8 @@ class CAMBdata(F2003Class):
         """
         Get comoving radial distance from us to redshift z in Mpc. This is efficient for arrays.
 
-        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions or power spectra.
+        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer
+        functions or power spectra.
 
         :param z: redshift
         :param tol: numerical tolerance parameter
@@ -1208,7 +1257,8 @@ class CAMBdata(F2003Class):
     def redshift_at_conformal_time(self, eta):
         """
         Convert conformal time array to redshift array.
-        Note that this function requires the transfers or background to have been calculated with no_thermo=False (the default).
+        Note that this function requires the transfers or background to have been calculated with no_thermo=False
+        (the default).
 
         :param eta: conformal time from bing bang (in Mpc), scalar or array
         :return: redshift at eta, scalar or array
@@ -1230,20 +1280,23 @@ class CAMBdata(F2003Class):
         """
         Get luminosity distance from to redshift z.
 
-        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions or power spectra.
+        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions or
+        power spectra.
 
         :param z: redshift or array of redshifts
         :return: luminosity distance (matches rank of z)
         """
 
-        if not np.isscalar(z): z = np.asarray(z)
+        if not np.isscalar(z):
+            z = np.asarray(z)
         return self.angular_diameter_distance(z) * (1.0 + z) ** 2
 
     def h_of_z(self, z):
         r"""
         Get Hubble rate at redshift z, in :math:`{\rm Mpc}^{-1}` units, scalar or array
 
-        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions or power spectra.
+        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions
+        or power spectra.
 
         Use hubble_parameter instead if you want in [km/s/Mpc] units.
 
@@ -1262,18 +1315,20 @@ class CAMBdata(F2003Class):
         """
         Get Hubble rate at redshift z, in km/s/Mpc units. Scalar or array.
 
-        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions or power spectra.
+        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions
+        or power spectra.
 
         :param z: redshift
         :return: H(z)/[km/s/Mpc]
         """
-        return constants.c * self.h_of_z(z) / 1e3
+        return (constants.c / 1e3) * self.h_of_z(z)
 
     def physical_time_a1_a2(self, a1, a2):
         """
         Get physical time between two scalar factors in Gigayears
 
-        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions or power spectra.
+        Must have called :meth:`calc_background`, :meth:`calc_background_no_thermo` or calculated transfer functions
+        or power spectra.
 
         :param a1: scale factor 1
         :param a2: scale factor 2
@@ -1363,7 +1418,8 @@ class CAMBdata(F2003Class):
 
     def cosmomc_theta(self):
         r"""
-        Get :math:`\theta_{\rm MC}`, an approximation of the ratio of the sound horizon to the angular diameter distance at recombination.
+        Get :math:`\theta_{\rm MC}`, an approximation of the ratio of the sound horizon to the angular diameter
+        distance at recombination.
 
         :return: :math:`\theta_{\rm MC}`
         """
